
//-----------------------------------------------------------------------------
// 1. DRIVER SETUP - create driver object, midi ports and detection information
//-----------------------------------------------------------------------------

var midiremote_api = require('midiremote_api_v1')
var deviceDriver = midiremote_api.makeDeviceDriver('Akai', 'MPK mini mk2', 'Darren Jones')

var midiInput = deviceDriver.mPorts.makeMidiInput();
var midiOutput = deviceDriver.mPorts.makeMidiOutput();

deviceDriver.makeDetectionUnit().detectPortPair(midiInput, midiOutput)
    .expectInputNameContains('MPK Mini Mk II')
    .expectOutputNameContains('MPK Mini Mk II')

deviceDriver.mOnActivate = function (activeDevice) {
    midiOutput.sendSysexFile(activeDevice, 'mpk_mk2_ram.syx', 5)
}

var surface = deviceDriver.mSurface

//----------------------------------------------------------------------------------------------------------------------
// 2. SURFACE LAYOUT - create control elements and midi bindings
//----------------------------------------------------------------------------------------------------------------------

function makeKnobStrip(knobIndex, x, y, ccNr) {
    var knobStrip = {}
    
    knobStrip.knob = surface.makeKnob(x * knobIndex + 16, y, 2, 2.3)
    knobStrip.knob.mSurfaceValue.mMidiBinding.setInputPort(midiInput).bindToControlChange(0, ccNr)

    return knobStrip
}

function makeTriggerPads(padIndex, x, y, ccNr) {
    var triggerPads = {}
    
    triggerPads.pad = surface.makeTriggerPad(x * padIndex + 3.5, y, 2.7, 2.7)
    triggerPads.pad.mSurfaceValue.mMidiBinding.setInputPort(midiInput).bindToControlChange(0, ccNr)

    return triggerPads
}

function makeSurfaceElements() {
    var surfaceElements = {}

    surfaceElements.numStrips = 8
    surfaceElements.knobStrips = {}

    var x = 2.3
    var y = 0.25
    for(var i = 0; i < surfaceElements.numStrips; ++i) {
        var knobIndex = i
        if (i > 3) {
            knobIndex = i - 4
            y = 2.65 
        } 
        var ccNr = i + 70
        
        surfaceElements.knobStrips[i] = makeKnobStrip(knobIndex, x, y, ccNr) 
    }

    x = 3
    y = 3
    surfaceElements.triggerPads = {}
    for(var i = 0; i < surfaceElements.numStrips; ++i) {
        var padIndex = i
        if (i > 3) {
            padIndex = i - 4
            y = 0
        } 
        var ccNr = i + 16
        
        surfaceElements.triggerPads[i] = makeTriggerPads(padIndex, x, y, ccNr)
    }

    surface.makePianoKeys(0, 7.1, 25, 6, 0, 24)

	var joyStickXY = surface.makeJoyStickXY(0.35, 0, 2, 2);
	joyStickXY.mX.mMidiBinding.setInputPort(midiInput).setIsConsuming(false).bindToPitchBend (0);
	joyStickXY.mY.mMidiBinding.setInputPort(midiInput).setIsConsuming(false).bindToControlChange (0, 1);

    // Lefthand Buttons
    surface.makeBlindPanel(0, 2.5, 1.3, 1)
    surface.makeBlindPanel(1.4, 2.5, 1.3, 1)
    surface.makeBlindPanel(0, 3.6, 1.3, 1)
    surface.makeBlindPanel(1.4, 3.6, 1.3, 1)
    surface.makeBlindPanel(0, 4.7, 1.3, 1)
    surface.makeBlindPanel(1.4, 4.7, 1.3, 1)

    // Pad Controls
    surface.makeBlindPanel(16, 4.75, 1.3, 1)
    surface.makeBlindPanel(18, 4.75, 1.3, 1)
    surface.makeBlindPanel(20, 4.75, 1.3, 1)
    surface.makeBlindPanel(22.5, 4.75, 1.3, 1)
    
    return surfaceElements
}

var surfaceElements = makeSurfaceElements()


//----------------------------------------------------------------------------------------------------------------------
// 3. HOST MAPPING - create mapping pages and host bindings
//----------------------------------------------------------------------------------------------------------------------

function makePageSelectedTrack() {
    var page = deviceDriver.mMapping.makePage('Selected Track')

    var selectedTrackChannel = page.mHostAccess.mTrackSelection.mMixerChannel
    for (var i = 0; i < 8; ++i) {
        page.makeValueBinding (surfaceElements.knobStrips[i].knob.mSurfaceValue, page.mHostAccess.mFocusedQuickControls.getByIndex(i)).setValueTakeOverModeJump()// setValueTakeOverModeScaled()//selectedTrackChannel.//mQuickControls.getByIndex(i)).setValueTakeOverModeScaled()
    }
    page.makeActionBinding (surfaceElements.triggerPads[0].pad.mSurfaceValue, page.mHostAccess.mTrackSelection.mAction.mNextTrack)
    page.makeValueBinding (surfaceElements.triggerPads[1].pad.mSurfaceValue, selectedTrackChannel.mValue.mSolo).setTypeToggle()
    page.makeValueBinding (surfaceElements.triggerPads[2].pad.mSurfaceValue, selectedTrackChannel.mValue.mEditorOpen).setTypeToggle()
    page.makeValueBinding (surfaceElements.triggerPads[3].pad.mSurfaceValue, page.mHostAccess.mTransport.mValue.mStart).setTypeToggle()
    page.makeActionBinding (surfaceElements.triggerPads[4].pad.mSurfaceValue, page.mHostAccess.mTrackSelection.mAction.mPrevTrack)
    page.makeValueBinding (surfaceElements.triggerPads[5].pad.mSurfaceValue, selectedTrackChannel.mValue.mMute).setTypeToggle()
    page.makeValueBinding (surfaceElements.triggerPads[6].pad.mSurfaceValue, selectedTrackChannel.mValue.mMonitorEnable).setTypeToggle()
    page.makeValueBinding (surfaceElements.triggerPads[7].pad.mSurfaceValue, page.mHostAccess.mTransport.mValue.mRecord).setTypeToggle()

    return page
}

//----------------------------------------------------------------------------------------------------------------------
// Construct Pages
//----------------------------------------------------------------------------------------------------------------------
var pageSelectedTrack = makePageSelectedTrack()

pageSelectedTrack.mOnActivate = function (context) {
	console.log('from script: AKAI MPKmini "Selected Track" page activated')
}
